/*  Motti -- a strategy game
    Copyright (C) 1999 Free Software Foundation

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef XWIN_H

#define XWIN_H 1

#define NBUTS 3

#include "map.h"
#include "occupy.h"
#include "xinit.h"

enum button_class {
  ATT, DEF, GUE
};

extern void x_update_buttons (struct x_info_struct *);
extern void x_draw_turn_win (struct x_info_struct *);
extern void x_draw_map (struct x_info_struct *, Coord, Coord);
extern void x_draw_effects (struct x_info_struct *, Action);
extern void translate_box (struct x_info_struct *, int, int, int, int,
			   Coord *, Coord *);
extern Coord win_pos2map_coord (struct x_info_struct *, int, int);
extern void x_create_end_win (struct x_info_struct *);
extern void x_draw_end_win (struct x_info_struct *);

#endif /* xwin.h  */
